@@ -11,18 +11,18 @@ findNFirstNonNull <- function(dataSet, N){
   if (length(dataSet) <= N){
     return(dataSet[!is.null(dataSet) & !is.na(dataSet) & dataSet != ""])
   }
-  maxMultiple = round(log(length(dataSet)) / log(10) + 1)
-  result = NULL
+  maxMultiple <- round(log(length(dataSet)) / log(10) + 1)
+  result <- NULL
   
   ## Computation
   for ( mult in 1:maxMultiple){
-    I <- (10^(mult - 1)):min(10^mult - 1, nrow(dataSet))
+    I <- (10 ^ (mult - 1)):min(10 ^ mult - 1, nrow(dataSet))
     data_sample <- dataSet[I]
     
     data_sample <- data_sample[!is.null(data_sample) & !is.na(data_sample) & data_sample != ""]
     
     if (length(data_sample) > 0 ){
-      result <- c(result, data_sample[1:min(N-length(result), length(data_sample))])
+      result <- c(result, data_sample[1:min(N - length(result), length(data_sample))])
     }
     
     
@@ -66,6 +66,22 @@ checkAndReturnDataTable <- function(dataSet, name = "dataSet"){
   return(dataSet)
 }
 
+
+###################################################################################################
+########################################## Control verbose ########################################
+###################################################################################################
+
+is.verbose <- function(verbose, function_name = "is.verbose"){
+  if (! is.logical(verbose)){
+    stop(function_name, " verbose should be logical (TRUE or FALSE).")
+  }
+}
+is.verbose_levels <- function(verbose, function_name = "is.verbose", max_level = 2){
+  if (! is.logical(verbose) & ! verbose %in% 1:max_level){
+    stop(function_name, " verbose should be logical (TRUE or FALSE) or an integer lower than ", max_level, ".")
+  }
+}
+
 ###################################################################################################
 ########################################## check if col is in dataset #############################
 ###################################################################################################
@@ -80,9 +96,9 @@ is.col <- function(dataSet, cols = NULL, ...){
   # Arguments for log
   args <- list(...)
   if (length(args) > 0){
-  	if (!is.null(args[["function_name"]])){
-  	  function_name <- args[["function_name"]]
-  	}
+    if (!is.null(args[["function_name"]])){
+      function_name <- args[["function_name"]]
+    }
   }
   else{
     function_name <- "is.col"
@@ -136,13 +152,13 @@ printl <- function(...){
 # Use when you build a progress bar for colnames
 #' @importFrom tcltk tkProgressBar setTkProgressBar
 initPB <- function(function_name, cols_names){
-	pb <- tkProgressBar(title = paste0(function_name, ": 0% done"), min = 1, max = length(cols_names)) # Construction d'une progress bar
-	pb$function_name = function_name
-	pb$cols_names = cols_names
-	return(pb)
+  pb <- tkProgressBar(title = paste0(function_name, ": 0% done"), min = 1, max = length(cols_names)) # Construction d'une progress bar
+  pb$function_name <- function_name
+  pb$cols_names <- cols_names
+  return(pb)
 }
 setPB <- function(pb, col){
-	setTkProgressBar(pb, which(pb$cols_names == col), title=paste(pb$function_name, ": ", round(which(pb$cols_names == col) / length(pb$cols_names) * 100, 0), "% done")) 
+  setTkProgressBar(pb, which(pb$cols_names == col), title=paste(pb$function_name, ": ", round(which(pb$cols_names == col) / length(pb$cols_names) * 100, 0), "% done")) 
 }
 
 ###################################################################################################
@@ -162,11 +178,9 @@ setPB <- function(pb, col){
 control_nb_rows <- function(dataSet, nb_rows, function_name = "", variable_name = "nb_rows"){
   ## Sanity check
   if (! is.numeric(nb_rows)){
-    stop(paste0(function_name,": ", variable_name," should be a numeric"))
-  }
-  if (! any(class(dataSet) %in% c("data.table", "data.frame", "matrix"))){
-  stop("checkIfIsColumn: dataSet should be a data.table, data.frame or matrix")
+    stop(paste0(function_name,": ", variable_name," should be a numeric."))
   }
+  dataSet <- checkAndReturnDataTable(dataSet)
   
   ## Initialization
   nb_rows <- round(nb_rows)
@@ -175,13 +189,13 @@ control_nb_rows <- function(dataSet, nb_rows, function_name = "", variable_name
   if (nb_rows > nrow(dataSet)){
     nb_rows <- nrow(dataSet)
     warning(paste0(function_name,": You want to check more rows than there are in dataSet, I set ", 
-                   variable_name," to ", nb_rows))
+                   variable_name," to ", nb_rows, "."))
   }
   if (nb_rows < 1){
     nb_rows <- min(30, nrow(dataSet))
     warning(paste0(function_name, 
                    ": You want to check at least a few rows than there are in dataSet, I set ", 
-                   variable_name," to ", nb_rows))
+                   variable_name," to ", nb_rows, "."))
   } 
   
   ## Wrapp-up
@@ -202,13 +216,13 @@ true.aggFunction <- function(functions, function_name = "true.aggFunction "){
       warning(paste0(function_name, ": ", fun, " is not a function, it wont be used."))
       functions <- functions[names(functions) != fun]
     }
-	else{
-	  # check aggregation
-	  if (length(functions[[fun]](1:3)) != 1){
-		warning(paste0(function_name, ": ", fun, " is not an aggegaration function, it wont be used. An aggregation function is a function that for multiple input return only one, exemple: sum."))
-		functions <- functions[names(functions) != fun]
-	  }
-	}
+    else{
+      # check aggregation
+      if (length(functions[[fun]](1:3)) != 1){
+        warning(paste0(function_name, ": ", fun, " is not an aggegaration function, it wont be used. An aggregation function is a function that for multiple input return only one, exemple: sum."))
+        functions <- functions[names(functions) != fun]
+      }
+    }
   }
   # Wrapp-up
   return(functions)
@@ -227,8 +241,9 @@ true.aggFunction <- function(functions, function_name = "true.aggFunction "){
 # @param object_name for log, object name when called (character)
 # @param type what type of output is expected for built function (character, numeric or logical)
 # @return A function
-function.maker <- function(object, function_name = "function.maker",  object_name, type){
+function.maker <- function(object, type, function_name = "function.maker",  object_name = "object"){
   built_function <- NULL
+  
   # If it's a constant
   if (any(class(object) %in% c("numeric", "integer", "factor", "logical", "date", "character"))){
     built_function <- function(...){return(object)}
@@ -237,6 +252,7 @@ function.maker <- function(object, function_name = "function.maker",  object_nam
     built_function <- object
   }
   
+  
   if (!is.null(built_function)){
     if (type == "numeric"){
       if (length(built_function(1:3)) == 1){
@@ -244,7 +260,7 @@ function.maker <- function(object, function_name = "function.maker",  object_nam
           stop(paste0(function_name, ": ", object_name, " should be or should return a numeric."))
         }
         if (is.na(built_function(c(1, NA)))){
-          warning(paste0(function_name, ": ", object_name, " is not handling NAs, it won't do anything"))
+          warning(paste0(function_name, ": ", object_name, " is not handling NAs, it won't do anything."))
         }
         return(built_function)  
       }
@@ -253,10 +269,10 @@ function.maker <- function(object, function_name = "function.maker",  object_nam
     if (type == "logical"){
       if (length(built_function(c(TRUE, FALSE))) == 1){
         if (!is.logical(built_function(c(TRUE, FALSE)))){ 
-          stop(paste0(function_name, ": ", object_name, " should be or should return a logical"))
+          stop(paste0(function_name, ": ", object_name, " should be or should return a logical."))
         }
         if (is.na(built_function(c(TRUE, NA)))){
-          warning(paste0(function_name, ": ", object_name, " is not handling NAs, it won't do anything"))
+          warning(paste0(function_name, ": ", object_name, " is not handling NAs, it won't do anything."))
         }
         return(built_function)  
       }
@@ -264,10 +280,10 @@ function.maker <- function(object, function_name = "function.maker",  object_nam
     if (type == "character"){
       if (length(built_function(c("a", "b"))) == 1){
         if (!is.character(built_function(c("a", "b")))){ 
-          stop(paste0(function_name, ": ", object_name, " should be or should return a character"))
+          stop(paste0(function_name, ": ", object_name, " should be or should return a character."))
         }
         if (is.na(built_function(c("a", NA)))){
-          warning(paste0(function_name, ": ", object_name, " is not handling NAs, it won't do anything"))
+          warning(paste0(function_name, ": ", object_name, " is not handling NAs, it won't do anything."))
         }
         return(built_function)  
       }
@@ -278,3 +294,63 @@ function.maker <- function(object, function_name = "function.maker",  object_nam
     stop(paste0(object_name, ": is in a shape that isn't handled, please provide constant or aggregation function."))
   }
 }
+
+
+
+#################################################################################################################
+##### make new col name #########################################################################################
+#################################################################################################################
+make_new_col_name <- function(new_col, col_names){
+  ## Working environement
+  function_name <- "make_new_col_name"
+  
+  ## Sanit check
+  if (! is.character(new_col) || ! is.character(col_names)){
+    stop(paste0(function_name, ": new_col and col_names should be character."))
+  }  
+  
+  ## Initialization
+  if (! new_col %in% col_names){
+	return(new_col)
+  }
+  i <- 1
+  while (paste0(new_col, i) %in% col_names){
+    i <- i + 1
+  }
+  return(paste0(new_col, i))
+}
+
+
+#################################################################################################################
+############################# build name separator ##############################################################
+#################################################################################################################
+build_name_separator <- function(args){
+  name_separator = "."
+  if (! is.null(args[["name_separator"]])){
+    if (is.character(args[["name_separator"]]) & length(args[["name_separator"]]) == 1){
+      name_separator <- args[["name_separator"]]
+    }
+    else{
+      stop("name_separator should be a character.")
+    }
+  }
+  return(name_separator)
+}
+
+
+#################################################################################################################
+############################# build factor_date_type ############################################################
+#################################################################################################################
+build_factor_date_type <- function(args){
+  factor_date_type = "yearmonth"
+  if (! is.null(args[["factor_date_type"]])){
+    if (is.character(args[["factor_date_type"]]) & length(args[["factor_date_type"]]) == 1){
+      name_separator <- args[["factor_date_type"]]
+    }
+    else{
+      stop("factor_date_type should be a character.")
+    }
+  }
+  return(factor_date_type)
+}
+
