@@ -5,8 +5,8 @@
 #' 
 #' Set as numeric a character column (or a list of columns) from a data.table
 #' @param dataSet Matrix, data.frame or data.table
-#' @param cols a list of colnames of dataSet (or just one) to transform into numerics
-#' @param stripString should i change ", " to "." in the string? (logical, default to FALSE) 
+#' @param cols list of column(s) name(s) of dataSet to transform into numerics
+#' @param stripString should I change "," to "." in the string? (logical, default to FALSE) 
 #' If set to TRUE, computation will be a bit longer
 #' @param verbose should the function log (logical, default to TRUE)
 #' @return  dataSet (as a \code{\link{data.table}}), with specified columns set as numeric. 
@@ -35,7 +35,9 @@ setColAsNumeric <- function(dataSet, cols, stripString = FALSE, verbose = TRUE){
   
   ## Sanity check
   dataSet <- checkAndReturnDataTable(dataSet)
-  cols = real_cols(cols, names(dataSet), function_name)
+  cols <- real_cols(cols, names(dataSet), function_name)
+  is.verbose(verbose)
+  
   ## Initialization
   
   ## Computation
@@ -45,14 +47,15 @@ setColAsNumeric <- function(dataSet, cols, stripString = FALSE, verbose = TRUE){
   for (col in cols){
     if (verbose){
       printl(function_name, ": I am doing the column", col)
+      options(warn = -1) # if verbose, disable warning, it will  be logged
     }
-    if ( (!class(dataSet[[col]])%in% c("character", "integer", "numeric")) & verbose){
-      warning(paste(function_name, ":", col, 
-					"isn\"t a character a numeric or an integer, i do nothing"))
+    if ( !(is.character(dataSet[[col]]) || is.numeric(dataSet[[col]])) & verbose){
+      warning(paste(function_name, ": ", col, 
+                    " isn't a character a numeric or an integer, i do nothing."))
     }
-    if (class(dataSet[[col]]) == "character"){
+    if (is.character(dataSet[[col]])){
       
-      nb_na_init = sum(is.na(dataSet[[col]]))
+      n_na_init <- sum(is.na(dataSet[[col]]))
       if (stripString){
         set(dataSet, NULL, col, as.numericStrip(dataSet[[col]]))
       }
@@ -60,11 +63,15 @@ setColAsNumeric <- function(dataSet, cols, stripString = FALSE, verbose = TRUE){
         set(dataSet, NULL, col, as.numeric(dataSet[[col]])) 
       }
       if (verbose){
-        printl(function_name, ":", sum(is.na(dataSet[[col]])) - nb_na_init, 
-			   "NA have been created due to transformation to numeric")
+        printl(function_name, ": ", sum(is.na(dataSet[[col]])) - n_na_init, 
+               " NA have been created due to transformation to numeric.")
       } 
     }
   }
+  if(verbose){
+    # reset warnings
+    options(warn = 0)
+  }
   
   ## Wrapp-up
   return(dataSet)
@@ -78,7 +85,7 @@ setColAsNumeric <- function(dataSet, cols, stripString = FALSE, verbose = TRUE){
 #'
 #' Set as character a column (or a list of columns) from a data.table
 #' @param dataSet Matrix, data.frame or data.table
-#' @param cols a list of colnames of dataSet (or just one) to transform into characters
+#' @param cols list of column(s) name(s) of dataSet to transform into characters
 #' @param verbose should the function log (logical, default to TRUE)
 #' @return  dataSet (as a \code{\link{data.table}}), with specified columns set as character. 
 #' @examples
@@ -95,9 +102,10 @@ setColAsCharacter <- function(dataSet, cols, verbose = TRUE){
   
   ## Sanity check
   dataSet <- checkAndReturnDataTable(dataSet)
+  is.verbose(verbose)
   
   ## Initialization
-  cols = real_cols(cols, names(dataSet), function_name)
+  cols <- real_cols(cols, names(dataSet), function_name)
   
   ## Computation
   if (verbose){
@@ -107,10 +115,10 @@ setColAsCharacter <- function(dataSet, cols, verbose = TRUE){
     if (verbose){
       printl(function_name, ": I am doing the column", col)
     }
-    if ( (class(dataSet[[col]]) == "character") & verbose){
-      printl(function_name, ":", col, "is a character, i do nothing")
+    if ( (is.character(dataSet[[col]])) & verbose){
+      printl(function_name, ": ", col, " is a character, i do nothing.")
     }
-    if (class(dataSet[[col]]) != "character"){
+    if (! (is.character(dataSet[[col]]))){
       set(dataSet, NULL, col, as.character(dataSet[[col]])) 
     }
     
@@ -125,12 +133,14 @@ setColAsCharacter <- function(dataSet, cols, verbose = TRUE){
 #'
 #' Set as POSIXct a character column (or a list of columns) from a data.table
 #' @param dataSet Matrix, data.frame or data.table
-#' @param cols a list of colnames of dataSet (or just one) to transform into dates
-#' @param format the format of date (function is faster if the format is provided) (default to NULL)
+#' @param cols list of column(s) name(s) of dataSet to transform into dates
+#' @param format date's format (function will be faster if the format is provided) 
+#' (character, default to NULL).\cr
+#' For timestamps, format need to be provided ("s" or "ms" or second or millisecond timestamps)
 #' @param verbose should the function log (logical, default to TRUE)
 #' @details 
 #' setColAsDate is way faster when format is provided. If you want to identify dates and format
-#' automatically, have a look to \code{\link{findAndTransformDates}}
+#' automatically, have a look to \code{\link{findAndTransformDates}}. \cr
 #' @return dataSet (as a \code{\link{data.table}}), with specified columns set as Date. 
 #' If the transformation generated only NA, the column is set back to its original value.
 #' @examples
@@ -145,6 +155,10 @@ setColAsCharacter <- function(dataSet, cols, verbose = TRUE){
 #' 
 #' # Control the results
 #' lapply(data_transformed,class)
+#' 
+#' # It also works with timestamps
+#' dataSet <- data.frame( time_stamp = c(1483225200, 1485990000, 1488495600))
+#' setColAsDate(dataSet, cols = "time_stamp", format = "s")
 #' @import data.table
 #' @importFrom lubridate parse_date_time 
 #' @importFrom stringr str_replace_all
@@ -155,33 +169,41 @@ setColAsDate <- function(dataSet, cols, format = NULL, verbose = TRUE){
   
   ## Sanity check
   dataSet <- checkAndReturnDataTable(dataSet)
+  is.verbose(verbose)
   
   ## Initialization
   start_time <- proc.time()
   if (verbose){
-    printl(function_name, ": I will set some columns as Date")
+    printl(function_name, ": I will set some columns as Date.")
   }
-  cols = real_cols(cols, names(dataSet), function_name)
+  cols <- real_cols(cols, names(dataSet), function_name)
+  n_transformed <- length(cols)
   
   ## Computation
   for (col in cols){
     if (verbose){
-      printl(function_name, ": I am doing the column", col)
+      printl(function_name, ": I am doing the column ", col, ".")
+      options(warn = -1) # if verbose, disable warning, it will  be logged
     }
-    if (class(dataSet[[col]]) != "character" & verbose){
-      warning(paste0(function_name, ": ", col, " is\'nt a character, i do nothing"))
-    }
-    if (class(dataSet[[col]]) == "character"){
-      nb_na_init <- sum(is.na(dataSet[[col]]))
+    if (is.character(dataSet[[col]])){
+      n_na_init <- sum(is.na(dataSet[[col]]))
       data_sample <- dataSet[[col]]
       # If format is NULL, we let R determine the format
       if (is.null(format)){
-        set(dataSet, NULL, col, as.POSIXct(dataSet[[col]])) 
+        # If format is not given, search for it. 
+        format_tmp <- identifyDates(dataSet[, c(col), with = FALSE], n_test = min(30, nrow(dataSet)))$formats
+        if (!is.null(format_tmp)){
+          set(dataSet, NULL, col, as.POSIXct(dataSet[[col]], format = format_tmp))
+        }
+        else{
+          printl(function_name, ":, ", col, " doesn't seem to be a date, if it really is please provide format.")
+        }
       }
       # If it isn't NULL
       if (!is.null(format)){
+        # it is faster if it's a format accepted by parse_date_time, so we check that
         format4parse_date_time <- formatForparse_date_time()
-		format_tmp = str_replace_all(format, "[[:punct:]]", "")
+        format_tmp <- str_replace_all(format, "[[:punct:]]", "")
         if (format_tmp %in% format4parse_date_time){
           set(dataSet, NULL, col, parse_date_time(dataSet[[col]], orders = format_tmp))
         }
@@ -190,82 +212,109 @@ setColAsDate <- function(dataSet, cols, format = NULL, verbose = TRUE){
         }
       }
       
-      nb_na_end <- sum(is.na(dataSet[[col]]))
+      n_na_end <- sum(is.na(dataSet[[col]]))
       if (verbose){
-        printl(function_name, ":", nb_na_end - nb_na_init, "NA have been created due to transformation to Date")
+        printl(function_name, ":", n_na_end - n_na_init, "NA have been created due to transformation to Date.")
       }
       # If we generated only NA and format wasn't provide, we shouldn't have changer it so we set it bakck to char
-      if (nb_na_end == nrow(dataSet) & nb_na_init < nrow(dataSet) & is.null(format)){
+      if (n_na_end == nrow(dataSet) & n_na_init < nrow(dataSet)){
         if (verbose){
-          printl(function_name, ":", "Since i generated only NAs i set", col, "as it was before")
+          printl(function_name, ":", "Since i generated only NAs i set ", col, " as it was before.")
         }
         dataSet[[col]] <- data_sample
       }
     }
+    else if (is.numeric(dataSet[[col]]) & !is.null(format)){
+      if (format == "s"){
+        set(dataSet, NULL, col, as.POSIXct(dataSet[[col]], origin = "1970-01-01 00:00:00"))
+      }
+      if (format == "ms"){
+        set(dataSet, NULL, col, as.POSIXct(dataSet[[col]] / 1000, origin = "1970-01-01 00:00:00"))
+      }
+    }
+    else{
+      options(warn = 0)
+      warning(paste0(function_name, ": I can't handle ", col, ", please see documentation."))
+      options(warn = -1)
+      n_transformed <- n_transformed - 1
+      next()
+    }
+    if (verbose){
+      # reset warnings
+      options(warn = 0)
+    }
   }
   
   ## Wrapp-up
   if (verbose){
     printl(function_name, ": it took me: ", round((proc.time() - start_time)[[3]], 2), 
-           "s to transform ", length(cols), " columns to Dates")
+           "s to transform ", n_transformed, " column(s) to Dates.")
   }
   return(dataSet)
-  
 }
 
 
-
 ############################################################################################################
 ########################################### charToFactorOrLogical ##########################################
 ############################################################################################################
 #' Set columns as factor
 #' 
-#' Set columns as factor, or logical if they have too many different values
+#' Set columns as factor and control number of unique element, to avoid having too large factors.
 #' @param dataSet Matrix, data.frame or data.table
-#' @param cols a list of colnames of dataSet (or just one) to transform into factor
-#' @param n_levels max number of levels for factor (integer, default to 53)
+#' @param cols list of column(s) name(s) of dataSet to transform into factor
+#' @param n_levels max number of levels for factor (integer, default to 53) 
+#' set it to -1 to disable control.
 #' @param verbose should the function log (logical, default to TRUE)
 #' @details 
-#' Control number of levels will help you to distinguish true categoricals from just characters 
-#' that should be handle in another way.
+#' Control number of levels will help you to distinguish true categorical columns from just characters 
+#' that should be handled in another way.
 #' @return dataSet (as a \code{\link{data.table}}), with specified columns set as factor or logical.
 #' @examples
 #' # Load messy_adult
 #' data("messy_adult")
 #' 
-#' # we wil change mail and education
-#' head(messy_adult[, .(mail, education)])
+#' # we wil change education
+#' messy_adult <- setColAsFactor(messy_adult, cols = "education")
 #' 
-#' messy_adult <- setColAsFactorOrLogical(messy_adult, cols = c("mail", "education"))
-#' 
-#' sapply(messy_adult[, .(mail, education)], class)
-#' head(messy_adult[, .(mail, education)])
-#' # education is now a factor and mail a logical wether there was or not an mail.
+#' sapply(messy_adult[, .(education)], class)
+#' # education is now a factor
 #' @export
-setColAsFactorOrLogical <- function(dataSet, cols, n_levels = 53, verbose = TRUE){
+setColAsFactor <- function(dataSet, cols, n_levels = 53, verbose = TRUE){
   ## Working environment
-  function_name = "setColAsFactorOrLogical"
+  function_name <- "setColAsFactor"
   
   ## Sanity check
   dataSet <- checkAndReturnDataTable(dataSet)
-  if (!is.numeric(n_levels)){stop(paste0(function_name, ": n_levels should be an integer"))}
-  cols = real_cols(cols, names(dataSet), function_name)
-  
+  if (!is.numeric(n_levels)){stop(paste0(function_name, ": n_levels should be an integer."))}
+  cols <- real_cols(cols, names(dataSet), function_name)
+  is.verbose(verbose)
+  if (verbose){
+	printl(function_name, ": I will set some columns to factor.")
+  }
+  n_transformed <- 0
+  start_time <- proc.time()
   ## Computation
   for (col in cols){ 
     if (verbose){
-      printl(function_name, ": I am doing the column", col)
+      printl(function_name, ": I am doing the column ", col, ".")
     }
-    if (fastMaxNbElt(dataSet[[col]], n_levels)){
-      set(dataSet, NULL, col, as.factor(dataSet[[col]]))
+    if (n_levels != -1){
+      if (fastMaxNbElt(dataSet[[col]], n_levels)){
+        set(dataSet, NULL, col, as.factor(dataSet[[col]]))
+		n_transformed <- n_transformed + 1
+      }
+      else{
+        warning(paste0(function_name, ": ", col, " has more than ", n_levels, " values, i don't transform it."))
+      }  
     }
     else{
-      set(dataSet, NULL, col, !is.na(dataSet[[col]]) & dataSet[[col]] != "" )
-	  
+      set(dataSet, NULL, col, as.factor(dataSet[[col]]))
     }
   }
-
-
+  if (verbose){
+    printl(function_name, ": it took me: ", round((proc.time() - start_time)[[3]], 2), 
+           "s to transform ", n_transformed, " column(s) to factor.")
+  }
   ## Wrapp up
   return(dataSet)
 }
